#!/usr/bin/env python2
# -*- coding: utf-8 -*-
###############################################################################
# conkyGmusicbrowser-GetCoverart.py is a simple python script to update a known
# cover art path to be refreshed on track change.
#
#  Author: VastOne
# Created: 01/28/2011
import traceback
import shutil
import gobject
import os
import sys
import eyeD3
import urllib

try:
    import dbus
    DBUS_AVAIL = True
except ImportError:
    # Dummy D-Bus library
    class _Connection:
        get_object = lambda *a: object()
    class _Interface:
        __init__ = lambda *a: None
        ListNames = lambda *a: []
    class Dummy: pass
    dbus = Dummy()
    dbus.Interface = _Interface
    dbus.service = Dummy()
    dbus.service.method = lambda *a: lambda f: f
    dbus.service.Object = object
    dbus.SessionBus = _Connection
    DBUS_AVAIL = False

class Gmusicbrowser_GetCoverart:

    COVERART_DESTINATION = "/tmp/gmusicbrowser-coverart.jpg"
    current = ""
    use_tag_image_only = False
    
    def __init__(self):
        
        try:
            print "conkygmusicbrowser-GetCoverart - Cover art will be copied to '%s'"%self.COVERART_DESTINATION
            print "* Initialising: Creating a D-Bus session..."
            self.bus = dbus.SessionBus()
            self.remote_player = self.bus.get_object('org.mpris.gmusicbrowser', '/Player')
            self.iface_player = dbus.Interface(self.remote_player, 'org.freedesktop.MediaPlayer')
            self.iface_player.connect_to_signal("TrackChange", self.OnTrackChange)
            print "* Listening: waiting for a track change..."
        except:
            print "** Error: Is gmusicbrowser running? The D-Bus service is not working."
            traceback.print_exc()
    
    def OnTrackChange(self,sender=None):
        try:
            
            self.props = self.iface_player.GetMetadata()
            
            if self.use_tag_image_only == True:
                src = None
            else:
                src = self.getCoverartPath()
            
            if src == None:
                
                print "** Track Changed: No cover art images known, Checking for embedded images in audio file..."
                
                if len(self.getCoverartImage(self.COVERART_DESTINATION)) > 0:
                    print "**                Image found and extracted to %s."%self.COVERART_DESTINATION
                else:
                    print "**                No image found."
                    
            elif src == self.current:
                
                print "** Track changed: Cover art unchanged, no copy necessary"

            elif os.path.exists(src) == True:
                
                print "** Track changed: Copying '%s' to '%s'"%(src,self.COVERART_DESTINATION)
                shutil.copy(src, self.COVERART_DESTINATION)
                self.current = src
                                    
            else:
                print "** Track changed: No sure what to do: src = '%s', dst='%s'"%(src,self.COVERART_DESTINATION) 
        except:
            print "*** Error: Begin"
            traceback.print_exc()
            print "*** Error: End"
        finally:
            print "* Listening: waiting for a track change..."
                        
    def getCoverartPath(self):
        
        try:       
            if "arturl" in self.props:
                #coverart = os.path.expanduser(str(self.props["arturl"])).replace("file://","")
                coverart = os.path.expanduser(urllib.unquote(str(self.props["arturl"])).replace("file://",""))
                #coverart = os.path.expanduser(str(props["arturl"]).replace(" ", "%20"))
                if coverart.find("http://") != -1:
                    coverart = None
            else:
                coverart = None
                
            return coverart
        except:
            traceback.print_exc()
            
    def getCoverartImage(self,savepath):

        # get file apth
        if "location" in self.props:
            location = self.props["location"]
        
        srcfilepath = urllib.unquote(location).replace("file://","")
        dstfolder = os.path.dirname(savepath) 
        dstfile = savepath.replace(os.path.dirname(savepath)+"/","")
        
        tag = eyeD3.Tag()
        
        if os.path.isfile(srcfilepath):
            if os.path.splitext(srcfilepath)[1] == ".mp3":
                # Extract picture
                try:
                    tag.link(srcfilepath)
                    for image in tag.getImages():
                        image.writeFile(dstfolder, dstfile)
                        return savepath
                except:
                    print "Unable to extract image for: " + srcfilepath
                    traceback.print_exc()
        return ""


if __name__ == '__main__':
    from dbus.mainloop.glib import DBusGMainLoop
    DBusGMainLoop(set_as_default=True)
    loop = gobject.MainLoop()
    Gmusicbrowser_GetCoverart()
    loop.run()
    
